# -*- coding: utf-8 -*-
#  Copyright 2011 Takeshi KOMIYA
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


def partial(func, *args, **keywords):
    def newfunc(*fargs, **fkeywords):
        newkeywords = keywords.copy()
        newkeywords.update(fkeywords)
        return func(*(args + fargs), **newkeywords)
    newfunc.func = func
    newfunc.args = args
    newfunc.keywords = keywords
    return newfunc


def wraps(src):
    def newfunc(dest):
        dest.__module__ = src.__module__
        dest.__name__ = src.__name__
        dest.__doc__ = src.__doc__
        d = getattr(src, '__dict__', {})
        if hasattr(dest, '__dict__'):
            dest.__dict__.update(d)
        else:
            dest.__dict__ = d.copy()
        return dest
    return newfunc



# -*- coding: utf-8 -*-

from blockdiag.parser import parse_string, ParseException
from blockdiag.parser import Graph, SubGraph, Statements, Node, Edge
from nose.tools import raises


def test_parser_basic():
    # basic digram
    code = """
           diagram test {
              A -> B -> C, D;
           }
           """

    tree = parse_string(code)
    assert isinstance(tree, Graph)


def test_parser_without_diagram_id():
    code = """
           diagram {
              A -> B -> C, D;
           }
           """
    tree = parse_string(code)
    assert isinstance(tree, Graph)

    code = """
           {
              A -> B -> C, D;
           }
           """
    tree = parse_string(code)
    assert isinstance(tree, Graph)


def test_parser_empty_diagram():
    code = """
           diagram {
           }
           """
    tree = parse_string(code)
    assert isinstance(tree, Graph)

    code = """
           {
           }
           """
    tree = parse_string(code)
    assert isinstance(tree, Graph)


def test_parser_diagram_includes_nodes():
    code = """
           diagram {
             A;
             B [label = "foobar"];
             C [color = "red"];
           }
           """
    tree = parse_string(code)
    assert isinstance(tree, Graph)
    assert len(tree.stmts) == 3
    assert isinstance(tree.stmts[0], Statements)
    assert isinstance(tree.stmts[0].stmts[0], Node)
    assert isinstance(tree.stmts[1], Statements)
    assert isinstance(tree.stmts[1].stmts[0], Node)
    assert isinstance(tree.stmts[2], Statements)
    assert isinstance(tree.stmts[2].stmts[0], Node)


def test_parser_diagram_includes_edges():
    code = """
           diagram {
             A -> B -> C;
           }
           """
    tree = parse_string(code)
    assert isinstance(tree, Graph)
    print tree.stmts
    assert len(tree.stmts) == 1
    assert isinstance(tree.stmts[0], Edge)

    code = """
           diagram {
             A -> B -> C [style = dotted];
             D -> E, F;
           }
           """
    tree = parse_string(code)
    assert isinstance(tree, Graph)
    print tree.stmts
    assert len(tree.stmts) == 2
    assert isinstance(tree.stmts[0], Edge)
    assert isinstance(tree.stmts[1], Edge)


def test_parser_diagram_includes_groups():
    code = """
           diagram {
             group {
               A; B;
             }
             group {
               C -> D;
             }
           }
           """
    tree = parse_string(code)
    assert isinstance(tree, Graph)
    assert len(tree.stmts) == 2

    assert isinstance(tree.stmts[0], SubGraph)
    assert len(tree.stmts[0].stmts) == 2
    assert isinstance(tree.stmts[0].stmts[0], Statements)
    assert isinstance(tree.stmts[0].stmts[0].stmts[0], Node)
    assert isinstance(tree.stmts[0].stmts[1], Statements)
    assert isinstance(tree.stmts[0].stmts[1].stmts[0], Node)

    assert isinstance(tree.stmts[1], SubGraph)
    assert len(tree.stmts[1].stmts) == 1
    assert isinstance(tree.stmts[1].stmts[0], Edge)


def test_parser_diagram_includes_diagram_attributes():
    code = """
           diagram {
             fontsize = 12;
             node_width = 80;
           }
           """
    tree = parse_string(code)
    assert isinstance(tree, Graph)
    assert len(tree.stmts) == 2


@raises(ParseException)
def test_parser_parenthesis_ness():
    code = ""
    parse_string(code)



# -*- coding: utf-8 -*-

from blockdiag.tests.utils import (stderr_wrapper, __build_diagram,
                                   __validate_node_attributes)


def test_single_edge_diagram():
    diagram = __build_diagram('single_edge.diag')

    assert len(diagram.nodes) == 2
    assert len(diagram.edges) == 1

    positions = {'A': (0, 0), 'B': (1, 0)}
    labels = {'A': 'A', 'B': 'B'}
    __validate_node_attributes('single_edge.diag', xy=positions, label=labels)


def test_two_edges_diagram():
    diagram = __build_diagram('two_edges.diag')

    assert len(diagram.nodes) == 3
    assert len(diagram.edges) == 2

    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0)}
    __validate_node_attributes('two_edges.diag', xy=positions)


def test_edge_shape():
    diagram = __build_diagram('edge_shape.diag')

    for edge in diagram.edges:
        if edge.node1.id == 'A':
            assert edge.dir == 'none'
        elif edge.node1.id == 'B':
            assert edge.dir == 'forward'
        elif edge.node1.id == 'C':
            assert edge.dir == 'back'
        elif edge.node1.id == 'D':
            assert edge.dir == 'both'


def test_edge_attribute():
    diagram = __build_diagram('edge_attribute.diag')

    for edge in diagram.edges:
        if edge.node1.id == 'D':
            assert edge.dir == 'none'
            assert edge.color == (0, 0, 0)
            assert edge.thick is None
        elif edge.node1.id == 'F':
            assert edge.dir == 'forward'
            assert edge.color == (0, 0, 0)
            assert edge.thick == 3
        else:
            assert edge.dir == 'forward'
            assert edge.color == (255, 0, 0)  # red
            assert edge.thick is None

    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0),
                 'D': (0, 1), 'E': (1, 1), 'F': (0, 2), 'G': (1, 2)}
    __validate_node_attributes('edge_attribute.diag', xy=positions)


def test_folded_edge_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0), 'D': (0, 1),
                 'E': (0, 2), 'F': (1, 1), 'Z': (0, 3)}
    __validate_node_attributes('folded_edge.diag', xy=positions)


def test_skipped_edge_right_diagram():
    filename = 'skipped_edge_right.diag'
    skipped = {('A', 'B'): False, ('A', 'C'): True}
    __validate_node_attributes(filename, edge_skipped=skipped)


def test_skipped_edge_rightup_diagram():
    filename = 'skipped_edge_rightup.diag'
    skipped = {('A', 'B'): False, ('D', 'C'): True}
    __validate_node_attributes(filename, edge_skipped=skipped)


def test_skipped_edge_rightdown_diagram():
    filename = 'skipped_edge_rightdown.diag'
    skipped = {('A', 'B'): False, ('A', 'C'): True}
    __validate_node_attributes(filename, edge_skipped=skipped)


def test_skipped_edge_up_diagram():
    filename = 'skipped_edge_up.diag'
    skipped = {('C', 'A'): True}
    __validate_node_attributes(filename, edge_skipped=skipped)


def test_skipped_edge_down_diagram():
    filename = 'skipped_edge_down.diag'
    skipped = {('A', 'C'): True}
    __validate_node_attributes(filename, edge_skipped=skipped)


def test_skipped_edge_leftdown_diagram():
    filename = 'skipped_edge_leftdown.diag'
    skipped = {('A', 'B'): False, ('C', 'G'): True}
    __validate_node_attributes(filename, edge_skipped=skipped)


@stderr_wrapper
def test_skipped_edge_flowchart_rightdown_diagram():
    filename = 'skipped_edge_flowchart_rightdown.diag'
    skipped = {('A', 'B'): False, ('A', 'D'): True}
    __validate_node_attributes(filename, edge_skipped=skipped)


@stderr_wrapper
def test_skipped_edge_flowchart_rightdown2_diagram():
    filename = 'skipped_edge_flowchart_rightdown2.diag'
    skipped = {('B', 'C'): False, ('A', 'C'): True}
    __validate_node_attributes(filename, edge_skipped=skipped)


def test_skipped_edge_portrait_right_diagram():
    filename = 'skipped_edge_portrait_right.diag'
    skipped = {('A', 'C'): True}
    __validate_node_attributes(filename, edge_skipped=skipped)


def test_skipped_edge_portrait_rightdown_diagram():
    filename = 'skipped_edge_portrait_rightdown.diag'
    skipped = {('A', 'B'): False, ('A', 'E'): True}
    __validate_node_attributes(filename, edge_skipped=skipped)


def test_skipped_edge_portrait_leftdown_diagram():
    filename = 'skipped_edge_portrait_leftdown.diag'
    skipped = {('A', 'B'): False, ('D', 'C'): True}
    __validate_node_attributes(filename, edge_skipped=skipped)


def test_skipped_edge_portrait_down_diagram():
    filename = 'skipped_edge_portrait_down.diag'
    skipped = {('A', 'B'): False, ('A', 'C'): True}
    __validate_node_attributes(filename, edge_skipped=skipped)


@stderr_wrapper
def test_skipped_edge_portrait_flowchart_rightdown_diagram():
    filename = 'skipped_edge_portrait_flowchart_rightdown.diag'
    skipped = {('A', 'B'): False, ('A', 'D'): True}
    __validate_node_attributes(filename, edge_skipped=skipped)


@stderr_wrapper
def test_skipped_edge_portrait_flowchart_rightdown2_diagram():
    filename = 'skipped_edge_portrait_flowchart_rightdown2.diag'
    skipped = {('B', 'C'): False, ('A', 'C'): True}
    __validate_node_attributes(filename, edge_skipped=skipped)



# -*- coding: utf-8 -*-

from blockdiag.tests.utils import __build_diagram, __validate_node_attributes

from blockdiag.utils.collections import defaultdict


def test_single_node_diagram():
    diagram = __build_diagram('single_node.diag')

    assert len(diagram.nodes) == 1
    assert len(diagram.edges) == 0
    assert diagram.nodes[0].label == 'A'
    assert diagram.nodes[0].xy == (0, 0)


def test_node_shape_diagram():
    shapes = {'A': 'box', 'B': 'roundedbox', 'C': 'diamond',
              'D': 'ellipse', 'E': 'note', 'F': 'cloud',
              'G': 'mail', 'H': 'beginpoint', 'I': 'endpoint',
              'J': 'minidiamond', 'K': 'flowchart.condition',
              'L': 'flowchart.database', 'M': 'flowchart.input',
              'N': 'flowchart.loopin', 'O': 'flowchart.loopout',
              'P': 'actor', 'Q': 'flowchart.terminator', 'R': 'textbox',
              'S': 'dots', 'T': 'none', 'U': 'square', 'V': 'circle',
              'Z': 'box'}
    __validate_node_attributes('node_shape.diag', shape=shapes)


def test_node_shape_namespace_diagram():
    shapes = {'A': 'flowchart.condition', 'B': 'condition', 'Z': 'box'}
    __validate_node_attributes('node_shape_namespace.diag', shape=shapes)


def test_node_has_multilined_label_diagram():
    positions = {'A': (0, 0), 'Z': (0, 1)}
    labels = {'A': "foo\nbar", 'Z': 'Z'}
    __validate_node_attributes('node_has_multilined_label.diag',
                               xy=positions, label=labels)


def test_quoted_node_id_diagram():
    positions = {'A': (0, 0), "'A'": (1, 0), 'B': (2, 0), 'Z': (0, 1)}
    __validate_node_attributes('quoted_node_id.diag', xy=positions)


def test_node_id_includes_dot_diagram():
    positions = {'A.B': (0, 0), 'C.D': (1, 0), 'Z': (0, 1)}
    __validate_node_attributes('node_id_includes_dot.diag', xy=positions)


def test_multiple_nodes_definition_diagram():
    positions = {'A': (0, 0), 'B': (0, 1), 'Z': (0, 2)}
    colors = {'A': (255, 0, 0), 'B': (255, 0, 0), 'Z': (255, 255, 255)}
    __validate_node_attributes('multiple_nodes_definition.diag', xy=positions,
                               color=colors)


def test_multiple_node_relation_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (1, 1),
                 'D': (2, 0), 'Z': (0, 2)}
    __validate_node_attributes('multiple_node_relation.diag', xy=positions)


def test_node_attribute():
    labels = {'A': 'B', 'B': 'double quoted', 'C': 'single quoted',
              'D': '\'"double" quoted\'', 'E': '"\'single\' quoted"',
              'F': 'F', 'G': 'G', 'H': 'H', 'I': 'I', 'J': 'Hello'}
    colors = {'A': (255, 0, 0), 'B': (255, 255, 255), 'C': (255, 0, 0),
              'D': (255, 0, 0), 'E': (255, 0, 0), 'F': (255, 255, 255),
              'G': (255, 255, 255), 'H': (255, 255, 255), 'I': (255, 255, 255),
              'J': (255, 255, 255)}
    textcolors = defaultdict(lambda: (0, 0, 0))
    textcolors['F'] = (255, 0, 0)
    numbered = defaultdict(lambda: None)
    numbered['E'] = '1'
    stacked = defaultdict(lambda: False)
    stacked['G'] = True
    fontsize = defaultdict(lambda: None)
    fontsize['H'] = 16
    linecolors = defaultdict(lambda: (0, 0, 0))
    linecolors['I'] = (255, 0, 0)
    orientations = defaultdict(lambda: 'horizontal')
    orientations['J'] = 'vertical'

    __validate_node_attributes('node_attribute.diag', label=labels,
                               color=colors, textcolor=textcolors,
                               numbered=numbered, stacked=stacked,
                               fontsize=fontsize, linecolor=linecolors,
                               label_orientation=orientations)


def test_node_height_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0),
                 'D': (2, 1), 'E': (1, 1), 'Z': (0, 2)}
    __validate_node_attributes('node_height.diag', xy=positions)


def test_branched_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0),
                 'D': (1, 1), 'E': (2, 1), 'Z': (0, 2)}
    __validate_node_attributes('branched.diag', xy=positions)


def test_multiple_parent_node_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (0, 2),
                 'D': (1, 2), 'E': (0, 1), 'Z': (0, 3)}
    __validate_node_attributes('multiple_parent_node.diag', xy=positions)


def test_twin_multiple_parent_node_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (0, 1),
                 'D': (1, 1), 'E': (0, 2), 'Z': (0, 3)}
    __validate_node_attributes('twin_multiple_parent_node.diag', xy=positions)


def test_flowable_node_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0), 'Z': (0, 1)}
    __validate_node_attributes('flowable_node.diag', xy=positions)


def test_plugin_autoclass_diagram():
    positions = {'A_emphasis': (0, 0), 'B_emphasis': (1, 0), 'C': (1, 1)}
    styles = {'A_emphasis': 'dashed', 'B_emphasis': 'dashed', 'C': None}
    colors = {'A_emphasis': (255, 0, 0), 'B_emphasis': (255, 0, 0),
              'C': (255, 255, 255)}

    __validate_node_attributes('plugin_autoclass.diag', xy=positions,
                               style=styles, color=colors)


def test_plugin_attributes_diagram():
    attr1 = {'A': "1", 'B': None}
    attr2 = {'A': "2", 'B': None}
    attr3 = {'A': "3", 'B': None}

    __validate_node_attributes('plugin_attributes.diag', test_attr1=attr1,
                               test_attr2=attr2, test_attr3=attr3)



# -*- coding: utf-8 -*-
import re
import sys
from StringIO import StringIO
from nose.tools import eq_
from blockdiag.builder import ScreenNodeBuilder
from blockdiag.parser import parse_string


def supported_pil():
    try:
        import _imagingft
        _imagingft

        return True
    except:
        return False


def with_pil(fn):
    if not supported_pil():
        fn.__test__ = False

    return fn


def supported_pdf():
    try:
        import reportlab
        reportlab

        return True
    except:
        return False


def with_pdf(fn):
    if not supported_pdf():
        fn.__test__ = False

    return fn


def argv_wrapper(func, argv=[]):
    def wrap(*args, **kwargs):
        try:
            argv = sys.argv
            sys.argv = []
            func(*args, **kwargs)
        finally:
            sys.argv = argv

    wrap.__name__ = func.__name__
    return wrap


def stderr_wrapper(func):
    def wrap(*args, **kwargs):
        try:
            stderr = sys.stderr
            sys.stderr = StringIO()

            print args, kwargs
            func(*args, **kwargs)
        finally:
            if sys.stderr.getvalue():
                print "---[ stderr ] ---"
                print sys.stderr.getvalue()

            sys.stderr = stderr

    wrap.__name__ = func.__name__
    return wrap


def assertRaises(exc):
    def decorator(func):
        def fn(self, *args, **kwargs):
            try:
                func(self, *args, **kwargs)
            except exc:
                pass
            else:
                msg = '%s does not raise exceptions: %s' % \
                      (func.__name__, str(exc))
                self.fail(msg)

        fn.__name__ = func.__name__
        return fn

    return decorator


def __build_diagram(filename):
    import os
    testdir = os.path.dirname(__file__)
    pathname = "%s/diagrams/%s" % (testdir, filename)

    code = open(pathname).read()
    tree = parse_string(code)
    return ScreenNodeBuilder.build(tree)


def __validate_node_attributes(filename, **kwargs):
    diagram = __build_diagram(filename)

    for name, values in kwargs.items():
        if re.match('edge_', name):
            print "[%s]" % name
            name = re.sub('edge_', '', name)
            for (id1, id2), value in values.items():
                found = False
                for edge in diagram.edges:
                    if edge.node1.id == id1 and edge.node2.id == id2:
                        print edge
                        eq_(value, getattr(edge, name))
                        found = True

                if not found:
                    raise RuntimeError('edge (%s -> %s) is not found' %
                                       (id1, id2))
        else:
            print "[node.%s]" % name
            for node in (n for n in diagram.nodes if n.drawable):
                print node
                value = getattr(node, name)
                eq_(values[node.id], value)



# -*- coding: utf-8 -*-

import os
import sys
import tempfile
import unittest2
from blockdiag.tests.utils import argv_wrapper, assertRaises, with_pdf

import blockdiag
from blockdiag.command import BlockdiagOptions
from blockdiag.utils.bootstrap import detectfont


class TestBootParams(unittest2.TestCase):
    def setUp(self):
        self.parser = BlockdiagOptions(blockdiag)

    @argv_wrapper
    def test_type_option_svg(self):
        sys.argv = ['', '-Tsvg', 'input.diag']
        options = self.parser.parse()
        self.assertEqual(options.output, 'input.svg')

        sys.argv = ['', '-TSVG', 'input.diag']
        options = self.parser.parse()
        self.assertEqual(options.output, 'input.svg')

        sys.argv = ['', '-TSvg', 'input.diag']
        options = self.parser.parse()
        self.assertEqual(options.output, 'input.svg')

        sys.argv = ['', '-TSvg', 'input.test.diag']
        options = self.parser.parse()
        self.assertEqual(options.output, 'input.test.svg')

    @argv_wrapper
    def test_type_option_png(self):
        sys.argv = ['', '-Tpng', 'input.diag']
        options = self.parser.parse()
        self.assertEqual(options.output, 'input.png')

    @with_pdf
    @argv_wrapper
    def test_type_option_pdf(self):
        sys.argv = ['', '-Tpdf', 'input.diag']
        options = self.parser.parse()
        self.assertEqual(options.output, 'input.pdf')

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_invalid_type_option(self):
        sys.argv = ['', '-Tsvgz', 'input.diag']
        self.parser.parse()

    @argv_wrapper
    def test_separate_option_svg(self):
        sys.argv = ['', '-Tsvg', '--separate', 'input.diag']
        self.parser.parse()

    @argv_wrapper
    def test_separate_option_png(self):
        sys.argv = ['', '-Tpng', '--separate', 'input.diag']
        self.parser.parse()

    @with_pdf
    @argv_wrapper
    def test_separate_option_pdf(self):
        sys.argv = ['', '-Tpdf', '--separate', 'input.diag']
        self.parser.parse()

    @argv_wrapper
    def test_svg_ignore_pil_option(self):
        sys.argv = ['', '-Tsvg', '--ignore-pil', 'input.diag']
        self.parser.parse()

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_png_ignore_pil_option(self):
        sys.argv = ['', '-Tpng', '--ignore-pil', 'input.diag']
        self.parser.parse()

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_pdf_ignore_pil_option(self):
        sys.argv = ['', '-Tpdf', '--ignore-pil', 'input.diag']
        self.parser.parse()

    @argv_wrapper
    def test_svg_nodoctype_option(self):
        sys.argv = ['', '-Tsvg', '--nodoctype', 'input.diag']
        self.parser.parse()

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_png_nodoctype_option(self):
        sys.argv = ['', '-Tpng', '--nodoctype', 'input.diag']
        self.parser.parse()

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_pdf_nodoctype_option(self):
        sys.argv = ['', '-Tpdf', '--nodoctype', 'input.diag']
        self.parser.parse()

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_svg_notransparency_option(self):
        sys.argv = ['', '-Tsvg', '--no-transparency', 'input.diag']
        self.parser.parse()

    @argv_wrapper
    def test_png_notransparency_option(self):
        sys.argv = ['', '-Tpng', '--no-transparency', 'input.diag']
        self.parser.parse()

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_pdf_notransparency_option(self):
        sys.argv = ['', '-Tpdf', '--no-transparency', 'input.diag']
        self.parser.parse()

    @argv_wrapper
    def test_config_option(self):
        try:
            tmp = tempfile.mkstemp()
            sys.argv = ['', '-c', tmp[1], 'input.diag']
            self.parser.parse()
        finally:
            os.close(tmp[0])
            os.unlink(tmp[1])

    @argv_wrapper
    def test_config_option_with_bom(self):
        try:
            tmp = tempfile.mkstemp()
            fp = os.fdopen(tmp[0], 'wt')
            fp.write("\xEF\xBB\xBF[blockdiag]\n")
            fp.close()

            sys.argv = ['', '-c', tmp[1], 'input.diag']
            self.parser.parse()
        finally:
            os.unlink(tmp[1])

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_invalid_config_option(self):
        sys.argv = ['', '-c', '/unknown_config_file', 'input.diag']
        self.parser.parse()

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_invalid_dir_config_option(self):
        try:
            tmp = tempfile.mkdtemp()

            sys.argv = ['', '-c', tmp, 'input.diag']
            self.parser.parse()
        finally:
            os.rmdir(tmp)

    @argv_wrapper
    def test_config_option_fontpath(self):
        try:
            tmp = tempfile.mkstemp()
            config = '[blockdiag]\nfontpath = /path/to/font\n'
            os.fdopen(tmp[0], 'wt').write(config)

            sys.argv = ['', '-c', tmp[1], 'input.diag']
            options = self.parser.parse()
            self.assertEqual(options.font, ['/path/to/font'])
        finally:
            os.unlink(tmp[1])

    @argv_wrapper
    def test_exist_font_config_option(self):
        try:
            tmp = tempfile.mkstemp()

            sys.argv = ['', '-f', tmp[1], 'input.diag']
            options = self.parser.parse()
            self.assertEqual(options.font, [tmp[1]])
            fontpath = detectfont(options)
            self.assertEqual(fontpath, tmp[1])
        finally:
            os.unlink(tmp[1])

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_not_exist_font_config_option(self):
        sys.argv = ['', '-f', '/font_is_not_exist', 'input.diag']
        options = self.parser.parse()
        detectfont(options)

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_not_exist_font_config_option2(self):
        sys.argv = ['', '-f', '/font_is_not_exist',
                    '-f', '/font_is_not_exist2', 'input.diag']
        options = self.parser.parse()
        detectfont(options)

    @argv_wrapper
    def test_no_size_option(self):
        sys.argv = ['', 'input.diag']
        options = self.parser.parse()
        self.assertEqual(None, options.size)

    @argv_wrapper
    def test_size_option(self):
        sys.argv = ['', '--size', '480x360', 'input.diag']
        options = self.parser.parse()
        self.assertEqual([480, 360], options.size)

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_invalid_size_option1(self):
        sys.argv = ['', '--size', '480-360', 'input.diag']
        self.parser.parse()

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_invalid_size_option2(self):
        sys.argv = ['', '--size', '480', 'input.diag']
        self.parser.parse()

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_invalid_size_option3(self):
        sys.argv = ['', '--size', 'foobar', 'input.diag']
        self.parser.parse()

    @argv_wrapper
    def test_auto_font_detection(self):
        sys.argv = ['', 'input.diag']
        options = self.parser.parse()
        fontpath = detectfont(options)
        self.assertTrue(fontpath)

    @assertRaises(RuntimeError)
    @argv_wrapper
    def test_not_exist_fontmap_config(self):
        sys.argv = ['', '--fontmap', '/fontmap_is_not_exist', 'input.diag']
        options = self.parser.parse()
        fontpath = detectfont(options)
        self.assertTrue(fontpath)

    @assertRaises(RuntimeError)
    def test_unknown_image_driver(self):
        from blockdiag.drawer import DiagramDraw
        from blockdiag.elements import Diagram

        DiagramDraw('unknown', Diagram())



# -*- coding: utf-8 -*-

from blockdiag.tests.utils import __build_diagram, __validate_node_attributes


def test_nested_groups_diagram():
    positions = {'A': (0, 0), 'B': (0, 1), 'Z': (0, 2)}
    __validate_node_attributes('nested_groups.diag', xy=positions)


def test_nested_groups_and_edges_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0), 'Z': (0, 1)}
    __validate_node_attributes('nested_groups_and_edges.diag', xy=positions)


def test_empty_group_diagram():
    positions = {'Z': (0, 0)}
    __validate_node_attributes('empty_group.diag', xy=positions)


def test_empty_nested_group_diagram():
    positions = {'Z': (0, 0)}
    __validate_node_attributes('empty_nested_group.diag', xy=positions)


def test_empty_group_declaration_diagram():
    positions = {'A': (0, 0), 'Z': (0, 1)}
    __validate_node_attributes('empty_group_declaration.diag', xy=positions)


def test_simple_group_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (1, 1), 'Z': (0, 2)}
    __validate_node_attributes('simple_group.diag', xy=positions)


def test_group_declare_as_node_attribute_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0),
                 'D': (2, 1), 'E': (2, 2), 'Z': (0, 3)}
    __validate_node_attributes('group_declare_as_node_attribute.diag',
                               xy=positions)


def test_group_attribute():
    diagram = __build_diagram('group_attribute.diag')

    for node in (x for x in diagram.nodes if not x.drawable):
        node.color = 'red'
        node.label = 'group label'
        node.shape = 'line'


def test_merge_groups_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (0, 1),
                 'D': (1, 1), 'Z': (0, 2)}
    __validate_node_attributes('merge_groups.diag', xy=positions)


def test_node_attribute_and_group_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0), 'Z': (0, 1)}
    labels = {'A': 'foo', 'B': 'bar', 'C': 'baz', 'Z': 'Z'}
    colors = {'A': (255, 0, 0), 'B': '#888888', 'C': (0, 0, 255),
              'Z': (255, 255, 255)}
    __validate_node_attributes('node_attribute_and_group.diag',
                               xy=positions, label=labels, color=colors)


def test_group_sibling_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (1, 2), 'D': (2, 0),
                 'E': (2, 1), 'F': (2, 2), 'Z': (0, 3)}
    __validate_node_attributes('group_sibling.diag', xy=positions)


def test_group_order_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (1, 1), 'Z': (0, 2)}
    __validate_node_attributes('group_order.diag', xy=positions)


def test_group_order2_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (1, 1),
                 'D': (2, 1), 'E': (1, 2), 'F': (2, 2), 'Z': (0, 3)}
    __validate_node_attributes('group_order2.diag', xy=positions)


def test_group_order3_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0),
                 'D': (2, 1), 'E': (1, 2), 'Z': (0, 3)}
    __validate_node_attributes('group_order3.diag', xy=positions)


def test_group_children_height_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (1, 1), 'D': (1, 2),
                 'E': (2, 0), 'F': (2, 2), 'Z': (0, 3)}
    __validate_node_attributes('group_children_height.diag', xy=positions)


def test_group_children_order_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (1, 1), 'D': (1, 2),
                 'E': (2, 0), 'F': (2, 1), 'G': (2, 2), 'Z': (0, 3)}
    __validate_node_attributes('group_children_order.diag', xy=positions)


def test_group_children_order2_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (1, 1), 'D': (1, 2),
                 'E': (2, 1), 'F': (2, 0), 'G': (2, 2), 'Z': (0, 3)}
    __validate_node_attributes('group_children_order2.diag', xy=positions)


def test_group_children_order3_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (1, 1), 'D': (1, 2),
                 'E': (2, 0), 'F': (2, 1), 'G': (2, 2), 'Q': (0, 3),
                 'Z': (0, 4)}
    __validate_node_attributes('group_children_order3.diag', xy=positions)


def test_group_children_order4_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (1, 1), 'D': (1, 2),
                 'E': (2, 0), 'Z': (0, 3)}
    __validate_node_attributes('group_children_order4.diag', xy=positions)


def test_node_in_group_follows_outer_node_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0), 'Z': (0, 1)}
    __validate_node_attributes('node_in_group_follows_outer_node.diag',
                               xy=positions)


def test_group_id_and_node_id_are_not_conflicted_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (0, 1),
                 'D': (1, 1), 'Z': (0, 2)}
    __validate_node_attributes('group_id_and_node_id_are_not_conflicted.diag',
                               xy=positions)


def test_outer_node_follows_node_in_group_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0), 'Z': (0, 1)}
    __validate_node_attributes('outer_node_follows_node_in_group.diag',
                               xy=positions)


def test_large_group_and_node_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (1, 1), 'D': (1, 2),
                 'E': (1, 3), 'F': (2, 0), 'Z': (0, 4)}
    __validate_node_attributes('large_group_and_node.diag', xy=positions)


def test_large_group_and_node2_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0), 'D': (3, 0),
                 'E': (4, 0), 'F': (5, 0), 'Z': (0, 1)}
    __validate_node_attributes('large_group_and_node2.diag', xy=positions)


def test_large_group_and_two_nodes_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (1, 1), 'D': (1, 2),
                 'E': (1, 3), 'F': (2, 0), 'G': (2, 1), 'Z': (0, 4)}
    __validate_node_attributes('large_group_and_two_nodes.diag', xy=positions)


def test_group_height_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0),
                 'D': (2, 1), 'E': (1, 2), 'Z': (0, 3)}
    __validate_node_attributes('group_height.diag', xy=positions)


def test_multiple_groups_diagram():
    positions = {'A': (0, 0), 'B': (0, 1), 'C': (0, 2), 'D': (0, 3),
                 'E': (1, 0), 'F': (1, 1), 'G': (1, 2), 'H': (2, 0),
                 'I': (2, 1), 'J': (3, 0), 'Z': (0, 4)}
    __validate_node_attributes('multiple_groups.diag', xy=positions)


def test_multiple_nested_groups_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (1, 1), 'Z': (0, 2)}
    __validate_node_attributes('multiple_nested_groups.diag', xy=positions)


def test_group_works_node_decorator_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (3, 0),
                 'D': (2, 0), 'E': (1, 1), 'Z': (0, 2)}
    __validate_node_attributes('group_works_node_decorator.diag', xy=positions)


def test_nested_groups_work_node_decorator_diagram():
    positions = {'A': (0, 0), 'B': (0, 1), 'Z': (0, 2)}
    __validate_node_attributes('nested_groups_work_node_decorator.diag',
                               xy=positions)


def test_reversed_multiple_groups_diagram():
    positions = {'A': (3, 0), 'B': (3, 1), 'C': (3, 2), 'D': (3, 3),
                 'E': (2, 0), 'F': (2, 1), 'G': (2, 2), 'H': (1, 0),
                 'I': (1, 1), 'J': (0, 0), 'Z': (0, 4)}
    __validate_node_attributes('reverse_multiple_groups.diag', xy=positions)


def test_group_and_skipped_edge_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (2, 0),
                 'D': (3, 0), 'E': (1, 1), 'Z': (0, 2)}
    __validate_node_attributes('group_and_skipped_edge.diag', xy=positions)


def test_group_orientation_diagram():
    positions = {'A': (0, 0), 'B': (1, 0), 'C': (1, 1),
                 'D': (2, 1), 'Z': (0, 2)}
    __validate_node_attributes('group_orientation.diag', xy=positions)


def test_nested_group_orientation_diagram():
    positions = {'A': (0, 0), 'B': (0, 1), 'C': (1, 0), 'Z': (0, 2)}
    __validate_node_attributes('nested_group_orientation.diag', xy=positions)



# -*- coding: utf-8 -*-
#  Copyright 2011 Takeshi KOMIYA
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import math
import unicodedata
from blockdiag.utils import Size


def is_zenkaku(char):
    u"""Detect given character is Japanese ZENKAKU character

        >>> is_zenkaku(u"A")
        False
        >>> is_zenkaku(u"あ")
        True
    """
    char_width = unicodedata.east_asian_width(char)
    return char_width in u"WFA"


def zenkaku_len(string):
    u"""
    Count Japanese ZENKAKU characters from string

    >>> zenkaku_len(u"abc")
    0
    >>> zenkaku_len(u"あいう")
    3
    >>> zenkaku_len(u"あいc")
    2
    """
    return len([x for x in string if is_zenkaku(x)])


def hankaku_len(string):
    u"""Count non Japanese ZENKAKU characters from string

        >>> hankaku_len(u"abc")
        3
        >>> hankaku_len(u"あいう")
        0
        >>> hankaku_len(u"あいc")
        1
    """
    return len([x for x in string if not is_zenkaku(x)])


def string_width(string):
    u"""Measure rendering width of string.
        Count ZENKAKU-character as 2-point and non ZENKAKU-character as 1-point

        >>> string_width(u"abc")
        3
        >>> string_width(u"あいう")
        6
        >>> string_width(u"あいc")
        5
    """
    widthmap = {'Na': 1, 'N': 1, 'H': 1, 'W': 2, 'F': 2, 'A': 2}
    return sum(widthmap[unicodedata.east_asian_width(c)] for c in string)


def textsize(string, font):
    u"""Measure rendering size (width and height) of line.
        Returned size will not be exactly as rendered text size,
        Because this method does not use fonts to measure size.

        >>> from blockdiag.utils.fontmap import FontInfo
        >>> box = [0, 0, 100, 50]
        >>> font = FontInfo('serif', None, 11)
        >>> textsize(u"abc", font)
        Size(width=19, height=11)
        >>> textsize(u"あいう", font)
        Size(width=33, height=11)
        >>> textsize(u"あいc", font)
        Size(width=29, height=11)
        >>> font = FontInfo('serif', None, 24)
        >>> textsize(u"abc", font)
        Size(width=40, height=24)
        >>> font = FontInfo('serif', None, 18)
        >>> textsize(u"あいう", font)
        Size(width=54, height=18)
    """
    width = (zenkaku_len(string) * font.size +
             hankaku_len(string) * font.size * 0.55)

    return Size(int(math.ceil(width)), font.size)


def cached(fn):
    def func(self, *args, **kwargs):
        name = fn.__name__
        key = args + tuple(kwargs.values())

        if name not in self._method_cache:
            self._method_cache[name] = {}

        if key not in self._method_cache[name]:
            self._method_cache[name][key] = fn(self, *args, **kwargs)

        return self._method_cache[name][key]

    return func



# -*- coding: utf-8 -*-
#  Copyright 2011 Takeshi KOMIYA
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import re
from blockdiag.utils import urlutil

try:
    from PIL import Image
except ImportError:
    try:
        import Image
    except ImportError:
        class Image:
            @classmethod
            def open(cls, filename):
                return cls(filename)

            def __init__(self, filename):
                self.filename = filename

            @property
            def size(self):
                from blockdiag.utils import jpeg
                import png

                try:
                    size = jpeg.JpegFile.get_size(self.filename)
                except:
                    try:
                        if isinstance(self.filename, (str, unicode)):
                            content = open(self.filename, 'r')
                        else:
                            self.filename.seek(0)
                            content = self.filename
                        image = png.Reader(file=content).read()
                        size = (image[0], image[1])
                    except:
                        size = None

                if hasattr(self.filename, 'seek'):
                    self.filename.seek(0)

                return size

_image_size_cache = {}


def get_image_size(filename):
    if filename not in _image_size_cache:
        uri = filename
        if urlutil.isurl(filename):
            import cStringIO
            import urllib
            try:
                uri = cStringIO.StringIO(urllib.urlopen(filename).read())
            except:
                return None

        _image_size_cache[filename] = Image.open(uri).size

    return _image_size_cache[filename]


def calc_image_size(size, bounded):
    if bounded[0] < size[0] or bounded[1] < size[1]:
        if (size[0] * 1.0 / bounded[0]) < (size[1] * 1.0 / bounded[1]):
            size = (size[0] * bounded[1] / size[1], bounded[1])
        else:
            size = (bounded[0], size[1] * bounded[0] / size[0])

    return size


def color_to_rgb(color):
    import webcolors
    if color == 'none' or isinstance(color, (list, tuple)):
        rgb = color
    elif re.match('#', color):
        rgb = webcolors.hex_to_rgb(color)
    else:
        rgb = webcolors.name_to_rgb(color)

    return rgb



# -*- coding: utf-8 -*-

import os
import sys
import re
import tempfile
from blockdiag.tests.utils import argv_wrapper, stderr_wrapper
from blockdiag.tests.utils import with_pil, supported_pil, with_pdf

import blockdiag
import blockdiag.command


def get_fontpath():
    filename = "VL-PGothic-Regular.ttf"
    testdir = os.path.dirname(__file__)
    return "%s/truetype/%s" % (testdir, filename)


def extra_case(func):
    pathname = get_fontpath()

    if os.path.exists(pathname):
        func.__test__ = True
    else:
        func.__test__ = False

    return func


@argv_wrapper
@stderr_wrapper
def __build_diagram(filename, _format, args):
    testdir = os.path.dirname(__file__)
    diagpath = "%s/diagrams/%s" % (testdir, filename)
    fontpath = get_fontpath()

    try:
        tmpdir = tempfile.mkdtemp()
        tmpfile = tempfile.mkstemp(dir=tmpdir)
        os.close(tmpfile[0])

        sys.argv = ['blockdiag.py', '-T', _format, '-o', tmpfile[1], diagpath]
        if args:
            if isinstance(args[0], (list, tuple)):
                sys.argv += args[0]
            else:
                sys.argv += args
        if os.path.exists(fontpath):
            sys.argv += ['-f', fontpath]

        blockdiag.command.main()

        if re.search('ERROR', sys.stderr.getvalue()):
            raise RuntimeError(sys.stderr.getvalue())
    finally:
        for filename in os.listdir(tmpdir):
            os.unlink(tmpdir + "/" + filename)
        os.rmdir(tmpdir)


def diagram_files():
    testdir = os.path.dirname(__file__)
    pathname = "%s/diagrams/" % testdir

    skipped = ['errors',
               'white.gif']

    return [d for d in os.listdir(pathname) if d not in skipped]


def test_generator_svg():
    args = []
    if not supported_pil():
        args.append('--ignore-pil')

    for testcase in generator_core('svg', args):
        yield testcase


@with_pil
@extra_case
def test_generator_png():
    for testcase in generator_core('png'):
        yield testcase


@with_pdf
@extra_case
def test_generator_pdf():
    for testcase in generator_core('pdf'):
        yield testcase


def generator_core(_format, *args):
    for diagram in diagram_files():
        yield __build_diagram, diagram, _format, args

        if re.search('separate', diagram):
            _args = list(args) + ['--separate']
            yield __build_diagram, diagram, _format, _args

        if _format == 'png':
            _args = list(args) + ['--antialias']
            yield __build_diagram, diagram, _format, _args


@extra_case
@argv_wrapper
def not_exist_font_config_option_test():
    fontpath = get_fontpath()
    sys.argv = ['', '-f', '/font_is_not_exist', '-f', fontpath, 'input.diag']
    options = blockdiag.command.BlockdiagOptions(blockdiag).parse()
    blockdiag.command.detectfont(options)


@argv_wrapper
@stderr_wrapper
def svg_includes_source_code_tag_test():
    from xml.etree import ElementTree

    testdir = os.path.dirname(__file__)
    diagpath = "%s/diagrams/single_edge.diag" % testdir
    fontpath = get_fontpath()

    try:
        tmpdir = tempfile.mkdtemp()
        tmpfile = tempfile.mkstemp(dir=tmpdir)
        os.close(tmpfile[0])

        sys.argv = ['blockdiag.py', '-T', 'SVG', '-o', tmpfile[1], diagpath]
        if os.path.exists(fontpath):
            sys.argv += ['-f', fontpath]
        if not supported_pil():
            sys.argv += ['--ignore-pil']

        blockdiag.command.main()

        if re.search('ERROR', sys.stderr.getvalue()):
            raise RuntimeError(sys.stderr.getvalue())

        # compare embeded source code
        source_code = open(diagpath).read()
        tree = ElementTree.parse(tmpfile[1])
        desc = tree.find('{http://www.w3.org/2000/svg}desc')

        # strip spaces
        source_code = re.sub('\s+', ' ', source_code)
        embeded = re.sub('\s+', ' ', desc.text)
        assert source_code == embeded
    finally:
        for filename in os.listdir(tmpdir):
            os.unlink(tmpdir + "/" + filename)
        os.rmdir(tmpdir)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import os, sys
import pkg_resources

sys.path.insert(0, 'src')
import blockdiag


def is_installed(name):
    try:
        pkg_resources.get_distribution(name)
        return True
    except:
        return False


long_description = \
        open(os.path.join("src","README.txt")).read() + \
        open(os.path.join("src","TODO.txt")).read()

classifiers = [
    "Development Status :: 4 - Beta",
    "Intended Audience :: System Administrators",
    "License :: OSI Approved :: Apache Software License",
    "Programming Language :: Python",
    "Topic :: Software Development",
    "Topic :: Software Development :: Documentation",
    "Topic :: Text Processing :: Markup",
]

requires = ['setuptools',
            'funcparserlib',
            'webcolors']
deplinks = []

# For readthedocs.org
# http://read-the-docs.readthedocs.org/en/latest/faq.html#how-do-i-change-behavior-for-read-the-docs
# Find imaging libraries

if 'READTHEDOCS' in os.environ:
    requires.append('Pillow')
elif is_installed('PIL'):
    requires.append('PIL')
elif is_installed('Pillow'):
    requires.append('Pillow')
elif sys.platform == 'win32':
    requires.append('Pillow')
else:
    requires.append('PIL')


# only for Python2.6
if sys.version_info > (2, 6) and sys.version_info < (2, 7):
    requires.append('OrderedDict')


setup(
     name='blockdiag',
     version=blockdiag.__version__,
     description='blockdiag generate block-diagram image file from spec-text file.',
     long_description=long_description,
     classifiers=classifiers,
     keywords=['diagram','generator'],
     author='Takeshi Komiya',
     author_email='i.tkomiya at gmail.com',
     url='http://blockdiag.com/',
     download_url='http://pypi.python.org/pypi/blockdiag',
     license='Apache License 2.0',
     py_modules=['blockdiag_sphinxhelper'],
     packages=find_packages('src'),
     package_dir={'': 'src'},
     package_data = {'': ['buildout.cfg']},
     include_package_data=True,
     install_requires=requires,
     extras_require=dict(
         test=[
             'Nose',
             'pep8>=1.3',
             'unittest2',
         ],
         pdf=[
             'reportlab',
         ],
         rst=[
             'docutils',
         ],
     ),
     dependency_links=deplinks,
     test_suite='nose.collector',
     tests_require=['Nose','pep8'],
     entry_points="""
        [console_scripts]
        blockdiag = blockdiag.command:main

        [blockdiag_noderenderer]
        box = blockdiag.noderenderer.box
        square = blockdiag.noderenderer.square
        roundedbox = blockdiag.noderenderer.roundedbox
        diamond = blockdiag.noderenderer.diamond
        minidiamond = blockdiag.noderenderer.minidiamond
        mail = blockdiag.noderenderer.mail
        note = blockdiag.noderenderer.note
        cloud = blockdiag.noderenderer.cloud
        circle = blockdiag.noderenderer.circle
        ellipse = blockdiag.noderenderer.ellipse
        beginpoint = blockdiag.noderenderer.beginpoint
        endpoint = blockdiag.noderenderer.endpoint
        actor = blockdiag.noderenderer.actor
        flowchart.database = blockdiag.noderenderer.flowchart.database
        flowchart.input = blockdiag.noderenderer.flowchart.input
        flowchart.loopin = blockdiag.noderenderer.flowchart.loopin
        flowchart.loopout = blockdiag.noderenderer.flowchart.loopout
        flowchart.terminator = blockdiag.noderenderer.flowchart.terminator
        textbox = blockdiag.noderenderer.textbox
        dots = blockdiag.noderenderer.dots
        none = blockdiag.noderenderer.none

        [blockdiag_plugins]
        attributes = blockdiag.plugins.attributes
        autoclass = blockdiag.plugins.autoclass

        [blockdiag_imagedrawers]
        imagedraw_png = blockdiag.imagedraw.png
        imagedraw_svg = blockdiag.imagedraw.svg
        imagedraw_pdf = blockdiag.imagedraw.pdf
     """,
)




# -*- coding: utf-8 -*-
#  Copyright 2011 Takeshi KOMIYA
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import blockdiag.parser
import blockdiag.builder
import blockdiag.drawer
core = blockdiag

import blockdiag.utils.bootstrap
import blockdiag.utils.collections
import blockdiag.utils.fontmap
utils = blockdiag.utils

from blockdiag.utils.rst import nodes
from blockdiag.utils.rst import directives

# FIXME: obsoleted interface (keep for compatibility)
from blockdiag import command, parser, builder, drawer
from blockdiag.utils import collections
from blockdiag.utils.fontmap import FontMap
from blockdiag.utils.rst.directives import blockdiag, BlockdiagDirective

(command, parser, builder, drawer, collections,
 FontMap, blockdiag, BlockdiagDirective)
